//========================================================================================
// ContigQualityPanel.java
// ---------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------
// $Id: //CBT-depot/CompBioTools++/includes/Mer.hh#1 $
//----------------------------------------------------------------------------------------
//
//========================================================================================
//.......1.........2.........3.........4.........5.........6.........7.........8.........9
//


import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

//import CBT;

//-----------------------------------------------------------------------------
// * ContigQualityPanel
//-----------------------------------------------------------------------------
//
public class ContigQualityPanel extends JPanel implements ListSelectionListener {
	 
	 private Dimension size; // indicates size taken up by graphics
	 JPanel drawingArea;
	 Ace    mA;
	 int    mScale;
    int xaxis = 50;
	 
	 private int cNum =0;
	 private int lastCNum =-1;
	 
	 private final Color colors[] = {
		  Color.red, Color.blue, Color.green, Color.orange,
		  Color.cyan, Color.magenta, Color.darkGray, Color.yellow};
	 private final int color_n = colors.length;
	 
	 //--------------------------------
	 // * ContigQualityPanel
	 //--------------------------------
	 //
	 public ContigQualityPanel(final Ace a) {
		  mA = a;
		  setOpaque(true);
		  size = new Dimension(0,0);
		  
		  mScale = 25;
	 		
		  //Set up the drawing area.
		  drawingArea = new JPanel() 				
				{
					 protected void paintComponent(Graphics g) {
						  super.paintComponent(g);
						  
						  int cSize = mA.GetContigSize(cNum);
						  size.width = cSize/mScale;
						  size.height = xaxis+30;
						  setPreferredSize(size);

						  // If the selected contig has changed, recalculate
						  // the coverage vector. 
						  if (cNum != lastCNum){
								mA.ComputeQuality(cNum);
								mA.ComputeHQDescrepency(cNum);
								lastCNum = cNum;
						  }
						  
						  // Draw the X-axis and tick marks. 
						  g.setColor(Color.black);
						  for(int i = 0;i< cSize;i++){
								if ((i%mScale) == 0){
									 g.fillRect(i/mScale,xaxis,2,2);
								}

								if ((i%1000) == 0){
									 g.fillRect(i/mScale,xaxis-5,1,10);
								}

								if ((i%5000) == 0){
									 String s = ""+i;
									 g.fillRect(i/mScale,xaxis-5,2,13);
									 g.drawString(s,i/mScale-20,xaxis+20);
								}
						  }

						  // Draw the quality
						  g.setColor(Color.blue);
						  String label2 = "Quality --";
						  g.drawString(label2,0,xaxis+20);
						  for(int i = 0;i< cSize;i++){
								if ((i%mScale) == 0){
									 int q = a.GetQuality(i);
									 int scaledq = q/4;
									 
									 if (q < 20){
										  g.setColor(Color.red);
										  g.fillRect(i/mScale,xaxis-(scaledq),1,2);
										  g.setColor(Color.blue);
									 }else{
										  if (q < 40){
												g.setColor(Color.orange);
												g.fillRect(i/mScale,xaxis-(scaledq),1,2);
												g.setColor(Color.blue);
										  }else{										
												g.fillRect(i/mScale,xaxis-(scaledq),1,2);
										  }
									 }
								}								  
						  } 

						  // Draw the HQ descrepency
						  g.setColor(Color.green);
						  String label = "HQ Descrepency --";
						  g.drawString(label,0,xaxis+40);
						  for(int i = 0;i< cSize;i++){
								if ((i%mScale) == 0){
									 int d = a.GetHQDescrepency(i)*4;
									 g.fillRect(i/mScale,xaxis-d,1,2);								
								}
						  }       
					 }
				};
		  
		  drawingArea.setBackground(Color.white);
		  
		  //Put the drawing area in a scroll pane.
		  JScrollPane scroller = new JScrollPane(drawingArea);
		  scroller.setPreferredSize(new Dimension(300,100));
		  
		  //Layout this demo.
		  setLayout(new BorderLayout());
		  //		  add(instructionPanel, BorderLayout.NORTH);
		  add(scroller, BorderLayout.CENTER);
		  
		  //Let the scroll pane know to update itself
		  //and its scrollbars.
		  drawingArea.revalidate();
		  setVisible(true);
	 }

	 public void valueChanged(ListSelectionEvent e)
	 {

		  if (e.getValueIsAdjusting()) return;

		  JList theList = (JList)e.getSource();
		  if(theList.isSelectionEmpty()) return;
		  else{
			   cNum = theList.getSelectedIndex();
				
				int cSize = mA.GetContigSize(cNum);
				size.width = cSize/mScale;
				size.height = xaxis;
				setPreferredSize(size);
				
			   drawingArea.revalidate();
				drawingArea.repaint();
		  }
	 }
	 
	 //	 public Dimension getPreferredSize(){
	 //	  Dimension minsize = new Dimension(300,xaxis+30);
	 //	  return(minsize);
		  //return(size);
	 // }

	 public Dimension getMinimumSize(){
		  Dimension minsize = new Dimension(300,100);
		  return(minsize);
	 }
}

	

// ToDo:
//
// * Have some way to measure the size of features
//     - Two vertical bars that can be slid around?
//     - Just report size of read regions?  
//     - More fine scale (tic marks). 
//
// * Add scale chooser (menu? +/- button? Slider?)
//
// * Integrate in template pair display. 
//
// ========================================
//
// * Color template coverage by size of template?
// * Fix so that scroll bars update more automatically. 
// * Improve performance (esp. Ace reading). 
// 
// * Plot WGS/BAC separately.   
// * Plot unhappy templates (pairs where mate isn't where/orientation expect). 
//    - length = template length.
//    - below, sort of a negative scale.    
//    - bac/wgs separate.   
// * Mouse indicate depth.  
// * Plot Quality

// * Option for template depth or fly-over template depth.  


// * Plot high quality descrepencies.  (lower priority). 
 
// ============================================


// Getting test data ideas:
// 
//   Restrict to projects finished in 2001.  
// * Look at lowest numbered consed assembly.  
//   - Ideally prefilter to remove trimmed ones
//   - Get NCBI record see if/how trimmed.  
// Maybe 1000 projects not trimmed? 
//


// Template cov
// -  0
// ?  1
// 
// Read cov
// - 0
// ? significantly below/above expectation


// Unhappy templates
// ? 1
// - > 1 


// 

// ===================
// * Option to display BAC library separately or all together.  Partition by first 4 char. 


// I think I am seeing two kinds of false joins:
// 
// For one the template coverage crashes while the read coverage 
// goes up.  This suggests a repeat is bridging two unrelated 
// contigs. 
// 
// It is not obvious how or where to split this sort of misjoin. 

// For the other, the template coverage crashes just before the
// read coverage crashes.   Basically, there is a single read, 
// or very small number of reads, connecting two unrelated 
// contigs. 
// 
// Splitting this should be easy, simply remove the offending
// read in the intersection and 
