//========================================================================================
// main.java
// ---------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------
// $Id: //CBT-depot/CompBioTools++/includes/Mer.hh#1 $
//----------------------------------------------------------------------------------------
//
//========================================================================================
//.......1.........2.........3.........4.........5.........6.........7.........8.........9
//
//
import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

//import XmFormLayout;
//import CBT;
//import TemplatePlotPanel;
//import TemplateSpanPanel;
//import ContigListPane;
//import WGSCoverPanel;

public class TemplateViewer extends JFrame{

  BadPairPlotPanel    mBadTemplatePlot;
  TemplatePlotPanel    mTemplatePlot;
  TemplateSpanPanel    mTemplateSpan;
  ContigListPane       mContigList;
  static JProgressBar  progressBar;
  ContigQualityPanel   mQualityPlot;
  WGSCoverPanel        mWGSCoverPlot;

  static {
    // Figure out what platform we are on, OSX, Linux, Windows...
    // extract and load the appropriate native library out of the .jar file. 
    NativeLoader nl = new NativeLoader();
    nl.LoadLibrary("AceSketch");
  }  

  //--------------------------------
  // * TemplateViewer
  //--------------------------------
  //	 
  public TemplateViewer(String AceName,String QualName){
    super("bcm-ace-plots: "+AceName);

    addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {System.exit(0);}
      });
			  
    Ace a = new Ace();
    LoadWithProgress(AceName,a);

    // KJD Temporary kludge. No progress bar for now...
    // Read the quality if it was supplied.  
    if (QualName != ""){
      a.GetReadQuality(QualName);
    }

    // Setup the components
    mTemplatePlot = new TemplatePlotPanel(a);
    addComponentListener(mTemplatePlot);

    mBadTemplatePlot = new BadPairPlotPanel(a);
    addComponentListener(mBadTemplatePlot);

    mTemplateSpan = new TemplateSpanPanel(a);
    addComponentListener(mTemplateSpan);


    mQualityPlot = new ContigQualityPanel(a);
    mWGSCoverPlot = new WGSCoverPanel(a);

    mContigList = new ContigListPane(a);		
    mTemplatePlot.setVisible(true);
    mBadTemplatePlot.setVisible(true);
    mTemplateSpan.setVisible(true);
    mContigList.setVisible(true);
  
    mContigList.addListSelectionListener(mTemplatePlot);
    mContigList.addListSelectionListener(mBadTemplatePlot);
    mContigList.addListSelectionListener(mTemplateSpan);
    mContigList.addListSelectionListener(mQualityPlot);
    mContigList.addListSelectionListener(mWGSCoverPlot);

    // Get rid of the progress bar
    getContentPane().removeAll();
        

    JPanel plotPanel = new JPanel();

    plotPanel.setLayout(new BoxLayout(plotPanel,BoxLayout.Y_AXIS));
    plotPanel.add(mTemplatePlot);
    plotPanel.add(mBadTemplatePlot);
    plotPanel.add(mTemplateSpan);
    plotPanel.add(mWGSCoverPlot);
    plotPanel.add(mQualityPlot);
    plotPanel.revalidate();

    //JScrollPane plotScrollPane = new JScrollPane(plotPanel);
    //plotScrollPane.revalidate();
    //getContentPane().add(plotScrollPane, BorderLayout.CENTER);

    getContentPane().add(plotPanel, BorderLayout.CENTER);
    getContentPane().add(mContigList, BorderLayout.WEST);
		  
    setVisible(true);
    buildMenu();
    pack();
  }

  //--------------------------------
  // * LoadWithProgress
  //--------------------------------
  //	
  void LoadWithProgress(String AceName,Ace a){

    int WIDTH = 400, HEIGHT = 100;
    setSize(WIDTH, HEIGHT);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation(d.width/2 - WIDTH/2, d.height/2 - HEIGHT/2);

    int NumReads = a.Open(AceName);

    JPanel progressPanel = new JPanel(){
	public Insets getInsets() {
	  return new Insets(20,20,20,20);
	}};
		  
    progressPanel.setLayout(new BoxLayout(progressPanel, BoxLayout.Y_AXIS));
		  
    getContentPane().add(progressPanel, BorderLayout.CENTER);
		  
    progressBar = new JProgressBar();
    progressBar.setMaximum(NumReads);
    progressBar.setAlignmentX(CENTER_ALIGNMENT);
    progressBar.setMinimum(0);
    progressBar.setValue(0);
    progressBar.setStringPainted(true);
    progressBar.setString("Reading "+AceName);
    progressPanel.add(progressBar);
    setVisible(true);
		  
    int totalReads=0;
    int cReads; 
    int numContigs;
    do{
      cReads = a.ReadContigBatch();
      numContigs = a.ContigNum();
      numContigs++;
      if (cReads>0){
	totalReads += cReads;
	progressBar.setString(numContigs+ " contigs. "+totalReads+" reads.");
	progressBar.setValue(totalReads);
      }
    }while(cReads>0);
    progressBar.setString(numContigs+ " contigs. "+totalReads+" reads.");
    progressBar.setValue(totalReads);		  
    a.Close();		  
  }


  //--------------------------------
  // * buildMenu
  //--------------------------------
  //	 
  private void buildMenu() { // Opens buildMenu method definition 
    JMenuBar mb = new JMenuBar();
    JMenu menu = new JMenu("File");
    JMenuItem item = new JMenuItem("Exit"); 
		  		
    item.addActionListener(new ActionListener() { 
	public void actionPerformed(ActionEvent e) {System.exit(0);}
      }); // Closes addActionListener method 
		  
    menu.add(item);
    mb.add(menu);
    setJMenuBar(mb); 
  }
	 
  //--------------------------------
  // * main
  //--------------------------------
  //
  public static void main(String argv[]) 
  {

    // argv[0] is ace name
    // argv[1] is quality name (optional)

    if (argv.length < 1){
      System.err.println("Must supply an ace file name.");
    }else{				
      if (argv.length == 1){
	TemplateViewer tv = new TemplateViewer(argv[0],"");											   }else{
	TemplateViewer tv = new TemplateViewer(argv[0],argv[1]);			
      }
    }
  }
}




//				String Wdir = new String();
//				Wdir = System.getProperty("user.dir","xxx");
//				String path = Wdir+"/libCBT.jnilib";

//				System.out.println("TV");
//				System.load(path);


/*

* Show total number of BAC reads/ WGS reads in plot. 

*/

// TODO
//
//
/*

* Make single panel for plots, with a check-box selection to turn them on and off. 
* Need a way to easily add new plots. 
* Contig list needs to show size, possibly be sortable by size. 
(field show only contigs > x size). 

* Plot bad template pairs below line. 
* Have a key panel. 


* Look at gaab.ace, Contig 70.  There is a wgs-only tail with 
decent template coverage.  What is going on?  What is this sequence?



*/
